#!ruby -Ks
# -*- mode: ruby; Encoding: utf-8 -*-
# Last updated: <2013/05/10 11:26:22 +0900>
#
# DXRubyで、スプライト + BG表示 + ogg再生テスト
#
# - DXRuby 1.5系開発版を使用(フルスクリーン表示が改善)
# - ogg再生には Ayame/Ruby を使用。

require 'dxruby'
require './ayame'

# Ayame/Ruby のバージョンを記述することで処理を変える。
$ayame_ver = "0.0.1"
# $ayame_ver = "0.0.2"

class MySpr < Sprite
  def initialize(x,y,vx,vy,img)
    super
    @fpx = x
    @fpy = y
    @fpvx = vx
    @fpvy = vy
    @w = img.width
    @h = img.height
    @whf = @w / 2
    @hhf = @h / 2
    self.image = img
    self.x = (@fpx - @whf).to_i
    self.y = (@fpy - @hhf).to_i
  end

  def update
    @fpx += @fpvx
    @fpy += @fpvy
    self.x = (@fpx - @whf).to_i
    self.y = (@fpy - @hhf).to_i
    @fpvx *= -1 if self.x + @w >= $SCR_W or self.x <= 0
    @fpvy *= -1 if self.y + @h >= $SCR_H or self.y <= 0
  end
end

def main_loop
  $SCR_W = 640
  $SCR_H = 480
	fullscr = false

  # フルスクリーン/ウインドウ表示を設定
  if fullscr
    Window.windowed = false
  else
    Window.windowed = true
  end
  Window.fps = 60
  Window.resize($SCR_W, $SCR_H)
  Window.frameskip = false

  # フォント読み込み
  fontdt = [
    {
      'filename' => 'res/VeraMoBd.ttf',
      'fontname' => 'Bitstream Vera Sans Mono Bold',
      'size' => 18
    },
    {
      'filename' => 'res/LiberationMono-Bold.ttf',
      'fontname' => 'Liberation Mono Bold',
      'size' => 18
    },
  ]

  fk = 0
  Font.install(fontdt[fk]['filename'])
  fnt = Font.new(fontdt[fk]['size'], fontdt[fk]['fontname'])
  fyadd = fontdt[fk]['size'] + 6

  # BG初期化
  bg1 = Image.loadTiles('res/bg1.png', 16, 16)
  bg2 = Image.loadTiles('res/bg2.png', 16, 16)
  bgarr = []
  bgcnt = 0
  16.times do |by|
    bgarr[by] = []
    16.times do |bx|
      bgarr[by][bx] = bgcnt
      bgcnt += 1
    end
  end

  # スプライト発生
  pmax = 512
  img = Image.load('res/ufo.png')
  x = 320
  y = 240
  sprgroup = []
  pmax.times do |i|
    rad = (i * 360 / pmax) * Math::PI / 180
    dx = 3.0 * Math::cos(rad)
    dy = 3.0 * Math::sin(rad)
    sprgroup.push(MySpr.new(x,y,dx,dy,img))
  end

	# BGM再生開始
	if $ayame_ver == "0.0.1"
		bgm = Ayame.new("res/bgm32.ogg")
		bgm.prefetch
		bgm.predecode
		bgm.play(-1)
	else
		bgm = Ayame.new("res/bgm32.ogg")
		bgm.play(-1, 0)
	end

  frame = 0
  loadval = 0
	bgx = [0,0]
	bgy = [0,0]
  sprdraw = true
  mesdt = [
    "F2: Full/Wdw",
    "F3: SPR ON/OFF",
    "ESC,Q: EXIT"
  ]

  # メインループ
  Window.loop do

    break if Input.keyPush?(K_ESCAPE) or Input.keyPush?(K_Q)
		sprdraw = !sprdraw if Input.keyPush?(K_F3)
		
		if Input.keyPush?(K_F2)
			# フルスクリーン/ウインドウ表示切替
			fullscr = !fullscr
			if fullscr
				Window.windowed = false
			else
				Window.windowed = true
			end
			Window.fps = 60
			Window.resize($SCR_W, $SCR_H)
			Window.frameskip = false
		end

		# BG描画
		ang = Math::cos(frame * Math::PI / 180)
    bgx[0] = -64 + (64.0 * ang)
    bgx[1] = -128 + (128.0 * ang)
    bgy[0] = (bgy[0] + 2) % 512
    bgy[1] = (bgy[1] + 6) % 512
    Window.drawTile(0, 0, bgarr, bg1, -bgx[0], -bgy[0], 20, 15)
    Window.drawTile(0, 0, bgarr, bg2, -bgx[1], -bgy[1], 20, 15)

    # スプライト描画
    Sprite.update(sprgroup)
    Sprite.draw(sprgroup) if sprdraw

		# FPSその他を描画
		fx = 8
		fy = 8
		lloadval = Window.getLoad
		loadval = lloadval if loadval < lloadval
		rfps = "#{Window.real_fps} FPS / CPU #{loadval.to_i} %"
		Window.drawFont(fx, fy, rfps, fnt)
		fy += fyadd
		mesdt.each do |str|
			Window.drawFont(fx, fy, str, fnt)
			fy += fyadd
		end
		
		if $ayame_ver != "0.0.1"
			# Ayame.update # フェードアウト処理用
		end

    frame += 1
    loadval = 0 if (frame) % 60 == 0

  end
	
	bgm.stop
  bgm.dispose
end

main_loop
exit

